% In this program, one can solve the reaction system from the paper,
% fit the experimental data using k_sr (side reaction rate) as a fit
% parameter, and plot the data and the simulation/fit results

% THE SECTIONS SHOULD BE EXECUTED CONSECUTIVELY

%% DEFINE ALL PARAMETERS FOR SIMULATIONS %%
clear all

%%%%%%%%%%%%%% Define parameters for simulation %%%%%%%%%%%%

t_h = 16; % Time at melting stage [s]
t_c = 1784; % Time at cold (ligation) stage [s]

n = 10; % Number of cycles (one cycle is one time t_h and one time t_c)
m = 2; % Feed/dilute every m cycle  

modif = 0.0003; % Side reaction rate (starting value for the fitting) [1/s]
ligon = 0.0006; % Ligation rate [1/s]

soff_l = 4.76e-3; % off-rate for short oligos at cold stage (lower bound)
%soff_l = 9.42e-2; % off-rate for short oligos at cold stage (try for upper bound)
son_l = 0.4; % on-rate for short oligos at cold stage
loff_l = 5.69e-08; % off-rate for long oligos at cold stage (lower bound)
%loff_l = 1.78e-3; % off-rate for long oligos at cold stage (try for upper bound)
lon_l = 0.4; % on-rate for long oligos at cold stage
soff_h = 40; % off-rate for short oligos at melting stage
son_h = 0.4; % on-rate for short oligos at melting stage
loff_h = 40; % off-rate for long oligos at melting stage
lon_h = 0.4; % on-rate for long oligos at melting stage

dil = 0.00625; % Rate of dilution [1/s] 
feed = 0.625; % Rate of fresh fragment feeding [uM/s]

%% SOLVE THE SIMULATION WITH SIDE REACTIONS %%
% USES PARAMETERS DEFINED IN THE PREVIOUS SECTION

initial_conditions = [10 10 10 10 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0];
    % Array of initial conditions [uM] for 
    % a b A B a_star B_star ab BA abxBA abA abB BAa BAb abB_star BAa_star
    % a_bxBA abxB_A a_star_bxBA abxB_star_A axA bxB a_starxA bxB_star
    
options = odeset('RelTol',1e-1,'AbsTol', 1e-3);
[t_array, replicate_ab, all_a, all_b, all_A, all_B, aandab, a, a_star] = ...
    function_solve_system(t_h, t_c, n, m, initial_conditions, ...
    modif, ligon, soff_l, son_l, loff_l, lon_l, soff_h, son_h, loff_h, lon_h,...
    dil, feed, options);

% SOME EXTRA VARIABLES ARE PRODUCED BY THE SIMULATION 
% (all_a, all_b, all_A, all_B, aandab, a, a_star), WHICH COULD BE USED TO TEST
% THE SIMULATION

%% %%%%%%%%%%%%%% plot simulation and experimental data %%%%%%%%%%%%%

%%%%%%%%%%%%%%%% load experimental data %%%%%%%%%%%%%%%%%%%%%%%
load experimental_data.txt; 
t_exp = experimental_data(:,1);
conc_exp = experimental_data(:,2)/1000;

%%%%%%%%%%%%%%%% plot %%%%%%%%%%%%%%%%%%%%%%%
figure(1)
plot(t_array/3600,replicate_ab,'g-','LineWidth',4);
hold on
plot(t_exp,conc_exp,'bo','LineWidth',4)
legend('simulation','experimental data')
xlabel('time (hours)', 'Fontsize',15);
ylabel('concentration (\muM)', 'Fontsize',15);

%% %%%%%%%%%%%%%% Fitting data for the side reaction rate %%%%%%%%%%%%%%%%

modif=0.0003; % starting value for the side reaction rate

f = @(modif,t_exp)function_concentration_anonym(modif, t_h, t_c, n, m, ...
    initial_conditions, ...
    ligon, soff_l, son_l, loff_l, lon_l, soff_h, son_h, loff_h, lon_h,...
    dil, feed, options,t_exp);

[modifbest,presnorm,presidual,exitflag,output] = ...
    lsqcurvefit(f,modif,t_exp,conc_exp);

%% SOLVE AND PLOT THE SIMULATION WITH FITTED SIDE REACTION RATE %%

%%%%%%%%%%%%%%%%%% use the side reaction rate from the fit above %%%%%%%%%%
modif = modifbest;

%%%%%%%%%%%%%%%%% solve the system with this side reaction rate %%%%%%%%%%%
initial_conditions = [10 10 10 10 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0];
    % Array of initial conditions [uM] for 
    % a b A B a_star B_star ab BA abxBA abA abB BAa BAb abB_star BAa_star
    % a_bxBA abxB_A a_star_bxBA abxB_star_A axA bxB a_starxA bxB_star
options = odeset('RelTol',1e-1,'AbsTol', 1e-3);
[t_array, replicate_ab, all_a, all_b, all_A, all_B, aandab, a, a_star] = ...
    function_solve_system(t_h, t_c, n, m, initial_conditions, ...
    modif, ligon, soff_l, son_l, loff_l, lon_l, soff_h, son_h, loff_h, lon_h,...
    dil, feed, options);

%%%%%%%%%%%%%%%% load experimental data %%%%%%%%%%%%%%%%%%%%%%%
load experimental_data.txt; 
t_exp = experimental_data(:,1);
conc_exp = experimental_data(:,2)/1000;

%%%%%%%%%%%%%%%% plot %%%%%%%%%%%%%%%%%%%%%%%
figure(2)
plot(t_array/3600,replicate_ab,'g-','LineWidth',4);
hold on
plot(t_exp,conc_exp,'bo','LineWidth',4)
legend('fit','experimental data')
xlabel('time (hours)', 'Fontsize',15);
ylabel('concentration (\muM)', 'Fontsize',15);